%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Linearni usporadani - predikaty: less; funkce: succ, pred
%
fof(antisymmetry, axiom, (
    ![X,Y]: ((less(X,Y) & less(Y,X)) => (X = Y))
)).
fof(transitivity, axiom, (
    ![X,Y,Z]: ((less(X,Y) & less(Y, Z)) => less(X, Z))
)).
fof(totality, axiom, (
    ![X,Y]: (less(X,Y) | less(Y,X))
)).
fof(succ, axiom, (
    ![X]: (   (less(X,succ(X))) 
            & (![Y]: (less(Y,X) | less(succ(X), Y)))
        )
)).
fof(pred, axiom, (
    ![X]: ((
        (pred(succ(X)) = X)
      & (succ(pred(X)) = X)
    ))
)).

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Fyzikalni model nadrazi
%
% 2 vstupy, 2 vystupy, hrany z leva do prava:
%
% in1 |^ ---> s1 ----> out1
%                \  / 
%                 \/
%                 /\
%                /  \
% in2 |^ ---> s2 ----> out2
%
% Legenda: |^ ... semafor

% pohyb vlaku po uzlech nadrazi: predikat at
fof(at_in1, axiom, ![T,Train]: (
    at(succ(T), Train, in1) <=> ( 
        enter(T, Train, in1) | 
        (at(T, Train, in1) & ~open(T, in1))
    )
)).

fof(at_in2, axiom, ![T,Train]: (
    at(succ(T), Train, in2) <=> ( 
        enter(T, Train, in2) | 
        (at(T, Train, in2) & ~open(T, in2))
    )
)).

fof(at_s1, axiom, ![T,Train]: (
    at(succ(T), Train, s1) <=> (at(T, Train, in1) & open(T, in1))
)).

fof(at_s2, axiom, ![T, Train]: (
    at(succ(T), Train, s2) <=> (at(T, Train, in2) & open(T, in2))
)).

fof(at_out1, axiom, ![T,Train]: (
    at(succ(T), Train, out1) <=> (
        (at(T, Train, s1) & switch(T, s1) = out1) |
        (at(T, Train, s2) & switch(T, s2) = out1) 
    )
)).

fof(at_out2, axiom, ![T,Train]: (
    at(succ(T), Train, out2) <=> (
        (at(T, Train, s1) & switch(T, s1) = out2) |
        (at(T, Train, s2) & switch(T, s2) = out2) 
    )
)).

% vlak je v jednu chvili pouze na jednom miste
fof(at_uniq, axiom, ![T,Train,N1,N2]: (
    (at(T, Train, N1) & at(T, Train, N2)) => (N1 = N2)
)).

% vlak ktery je v nadrazi do nadrazi nevstupuje
fof(at_nondup, axiom, ![T,Train,N,OtherN]: (
    at(T, Train, N) => ~enter(T, Train, OtherN)
)).

% vlak nikdy nevstupuje na obsazeny uzel
fof(input_noncol, axiom, ![T,Train,OtherTrain,N]: (
    at(T, Train, N) => ~enter(T, OtherTrain, N)
)).

% 2 ruzne vlaky nevstupuju do stejneho uzlu v jeden okamzik
fof(enter_uniq, axiom, ![T,Train1,Train2,N]: (
    (enter(T, Train1, N) & enter(T, Train2, N)) => (Train1 = Train2)
)).

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Ridici system nadrazi
%

% tic/tac hodiny
fof(clock_dual, axiom, ![T]: (
    tic(T) <=> ~tac(T)
)).

fof(clock_change, axiom, ![T]: ( (T != succ(T)) =>
    (tic(succ(T)) <=> tac(T))
)).

% prepinani vyhybek: funkce switch
fof(switch_s1_out1, axiom, ![T]: (
    switch(succ(T), s1) = out1 <=> (?[Train]: (at(T, Train, s1) & gate(Train) = out1))
)).

fof(switch_s1_out2, axiom, ![T]: (
    switch(succ(T), s1) = out2 <=> (?[Train]: (at(T, Train, s1) & gate(Train) = out2))
)).

fof(switch_s2_out1, axiom, ![T]: (
    switch(succ(T), s2) = out1 <=> (?[Train]: (at(T, Train, s2) & gate(Train) = out1))
)).

fof(switch_s2_out2, axiom, ![T]: (
    switch(succ(T), s2) = out2 <=> (?[Train]: (at(T, Train, s2) & gate(Train) = out2))
)).

% definice: in_conflict(T) ... "v case T nastava konfliktni situace na vstupu"
fof(in_conflict, axiom, ![T]: (
    in_conflict(T) <=> (?[Train1, Train2]: ((Train1 != Train2) & 
        at(T, Train1, in1) & at(T, Train2, in2) & gate(Train1) = gate(Train2)
    ))
)).

% otevirani navestidel: predikat open
fof(open_in1, axiom, ![T]: (
    open(T, in1) <=> (tic(T) | ~in_conflict(T))
)).

fof(open_in2, axiom, ![T]: (
    open(T, in2) <=> (tac(T) | ~in_conflict(T))
)).

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Restrikce domeny symbolu
%
fof(at_restr, axiom, ![T,Train,N]: (
    at(T, Train, N) => ( (N=in1) | (N=in2) | (N=s1) | (N=s2) | (N=out1) | (N=out2) )
)).

fof(open_restr, axiom, ![T,N]: (
    open(T, N) => ( (N=in1) | (N=in2) )
)).

fof(gate_restr, axiom, ![Train]: (
    (gate(Train) = out1) | (gate(Train) = out2)
)).

fof(distinct_nodes, axiom, (
    (in1 != in2) & (in1 != s1) & (in1 != s2) & (in1 != out1) & (in1 != out2) 
                 & (in2 != s1) & (in2 != s2) & (in2 != out1) & (in2 != out2) 
                               & (s1 != s2)  & (s1 != out1)  & (s1 != out2)  
                                             & (s2 != out1)  & (s2 != out2)  
                                                             & (out1 != out2)
)).

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Definice pro dokazovani
%

% definice: safe(T,N) ... "v case T nenastava kolize v uzlu N"
fof(node_safe, axiom, ![T,N]: (safe(T,N) <=> 
    ![Train1,Train2]: (((at(T, Train1, N) & at(T, Train2, N)) => Train1 = Train2))
)).

% definice: notBlocked(N) ... "na vstupu N nikdy nezustane zablokovany vlak "
fof(not_blocked, axiom, ![N]: (notBlocked(N) <=> (
    ![T1,Train]: (
            ((at(T1, Train, N) & ~open(T1, N))) 
        => 
            (?[T2]: (less(T1, T2) & open(T2, N)))
    )
))). 

